
//
//  VertexVersion.cpp - Sample Vertex Version Demonstration
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//	This application uses the common classes used by the samples included with the
//	DirectX SDK.
//
//  Created - 22 Apr 2003 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries

#define STRICT

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <math.h>
#include <d3dx9.h>
#include <TChar.h>

	//	Microsoft DirectX SDK Helper Functions

#include "DXUtil.h"
#include "D3DEnumeration.h"
#include "D3DSettings.h"
#include "D3DApp.h"





//
//	Application Class
//
//	Inherits from the DirectX Samples Class 'CD3Application'. 
//

class CMyD3DApplication : public CD3DApplication
{

protected:
		//	Enumeration of Shader Device Objects
    HRESULT ConfirmDevice( D3DCAPS9* pCaps, DWORD dwBehavior, 
			D3DFORMAT adapterFmt, D3DFORMAT backBufferFmt );

public:
	FILE	*fp;											// Logging File
	BOOL	bDump;

	void DumpAdapterInfo(void);

	void DumpDeviceObj( D3DCAPS9* pCaps, DWORD dwBehavior, 
			D3DFORMAT adapterFmt, D3DFORMAT backBufferFmt );

    CMyD3DApplication();
};



//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication()
{
    m_strWindowTitle    = _T("VertexVersion");
    m_d3dEnumeration.AppUsesDepthBuffer   = TRUE;
	bDump = false;
}



//
//	Dump Adapter Information
//

void CMyD3DApplication::DumpAdapterInfo(void)
{
	HRESULT rval;
	uint nAdapter, nAdapterCnt;
	D3DADAPTER_IDENTIFIER9 info;

	nAdapterCnt = m_pD3D->GetAdapterCount();

	for (nAdapter = 0; nAdapter < nAdapterCnt; nAdapter++)
	{
		rval = m_pD3D->GetAdapterIdentifier( nAdapter, 0, &info);
		if (D3D_OK == rval)
		{
			fprintf( fp, "Adapter#%u  '%s'  %s  \"%s\"\n", nAdapter, info.Description,
						info.Driver, info.DeviceName);
		}
	}

	fputs("\n", fp);
}



static char *ImageFormat( char *p, D3DFORMAT dwFormat )
{
	switch(dwFormat)
	{
		case D3DFMT_R8G8B8:
			p += sprintf( p, "8:8:8 RGB" );
			break;

		case D3DFMT_A8R8G8B8:			// Alpha
			p += sprintf( p, "8:8:8:8 ARGB" );
			break;

		case D3DFMT_X8R8G8B8:
			p += sprintf( p, "8:8:8:8 _RGB" );
			break;

		case D3DFMT_R5G6B5:
			p += sprintf( p, "5:6:5 RGB" );
			break;

		case D3DFMT_X1R5G5B5:
			p += sprintf( p, "1:5:5:5 _RGB" );
			break;

		case D3DFMT_A1R5G5B5:
			p += sprintf( p, "1:5:5:5 ARGB" );
			break;

		case D3DFMT_A4R4G4B4:
			p += sprintf( p, "4:4:4:4 ARGB" );
			break;

		case D3DFMT_R3G3B2:
			p += sprintf( p, "3:3:2 RGB" );
			break;

		case D3DFMT_A8:
			p += sprintf( p, "8 A" );
			break;

		case D3DFMT_A8R3G3B2:
			p += sprintf( p, "8:3:3:2 ARGB" );
			break;

		case D3DFMT_X4R4G4B4:
			p += sprintf( p, "4:4:4:4 _RGB" );
			break;

		case D3DFMT_A2B10G10R10:
			p += sprintf( p, "2:10:10:10 ARGB" );
			break;

		case D3DFMT_A8B8G8R8:
			p += sprintf( p, "8:8:8:8 ABGR" );
			break;

		case D3DFMT_X8B8G8R8:
			p += sprintf( p, "8:8:8:8 ABGR" );
			break;

		case D3DFMT_G16R16:
			p += sprintf( p, "16:16 GR" );
			break;

		case D3DFMT_A2R10G10B10:
			p += sprintf( p, "1:10:10:10 ARGB" );
			break;

		case D3DFMT_A16B16G16R16:
			p += sprintf( p, "16:16:16:16 ARGB" );
			break;

		case D3DFMT_A8P8:
			p += sprintf( p, "8:8 Ai" );
			break;

		case D3DFMT_P8:
			p += sprintf( p, "8 i" );
			break;

		case D3DFMT_L8:
			p += sprintf( p, "8 Lum" );
			break;

		case D3DFMT_L16:
			p += sprintf( p, "16 Lum" );
			break;

		case D3DFMT_A8L8:
			p += sprintf( p, "8:8 A,Lum" );
			break;

		case D3DFMT_A4L4:
			p += sprintf( p, "4:4 A,Lum" );
			break;

		default:
			break;
	}

	return p;
}



//
//	Device Dump
//

void CMyD3DApplication::DumpDeviceObj( 
				D3DCAPS9* pCaps,			// Capabilities
				DWORD dwBehavior,						// Behavioral bits
                D3DFORMAT adapterFormat, D3DFORMAT backBufferFormat )
{
	char *p, szBuf[256 ];

	p = szBuf;

	p += sprintf( p, "#%u  Vertex: %u.%u  Pixel: %u.%u",
			 pCaps->AdapterOrdinal,
			 D3DSHADER_VERSION_MAJOR( pCaps->VertexShaderVersion ),
			 D3DSHADER_VERSION_MINOR( pCaps->VertexShaderVersion ),
			 D3DSHADER_VERSION_MAJOR( pCaps->PixelShaderVersion  ),
			 D3DSHADER_VERSION_MINOR( pCaps->PixelShaderVersion  ));

	switch( pCaps->DeviceType)
	{		//	Hardware rasterization, Software, Hardware, Mixed
		case D3DDEVTYPE_HAL:
			p += sprintf( p, " HAL " );
			break;

			//	D3D features implemented in software
		case D3DDEVTYPE_REF:
			p += sprintf( p, " REF " );
			break;

			//	Pluggable 'registered' software device
		case D3DDEVTYPE_SW:
			p += sprintf( p, " SW " );
			break;

		default:
			p += sprintf( p, " 0x%x ", pCaps->DeviceType );
			break;
	}

	if (D3DCREATE_SOFTWARE_VERTEXPROCESSING & dwBehavior)
	{
		p += sprintf( p, " _SOFTWARE_ " );
	}

	if (D3DCREATE_HARDWARE_VERTEXPROCESSING & dwBehavior)
	{
		p += sprintf( p, " _HARDWARE_ " );
	}

	if (D3DCREATE_MIXED_VERTEXPROCESSING & dwBehavior)
	{
		p += sprintf( p, " _MIXED_ " );
	}

	if (D3DCREATE_PUREDEVICE & dwBehavior)
	{
		p += sprintf( p, " _PURE_ " );
	}

	p += sprintf( p, " .MaxStreams:%u  .MaxVConst:%u ",
			 pCaps->MaxStreams, pCaps->MaxVertexShaderConst );	

	p = ImageFormat( p, adapterFormat );
	p += sprintf( p, "  " );
	p = ImageFormat( p, backBufferFormat );

	p += sprintf( p, "  2^n:%u  sq:%u  mm:%u  w:%u h:%u",
			(pCaps->TextureCaps & D3DPTEXTURECAPS_POW2) ? 1 : 0,
			(pCaps->TextureCaps & D3DPTEXTURECAPS_SQUAREONLY) ? 1 : 0,
			(pCaps->TextureCaps & D3DPTEXTURECAPS_MIPMAP) ? 1 : 0,
			pCaps->MaxTextureWidth, pCaps->MaxTextureHeight
			);

	p += sprintf( p, "\n" );

	fputs( szBuf, fp );

	OutputDebugString( szBuf );
}



//
//	Enumeration of Devices
//
//	Called from Common\Src\D3DApp.cpp - ConfirmDeviceHelper()
//
//	The devices we wish can be selected here by the return of S_OK.
//
//	NOTE: Normally wouldn't print these out, but doing so here for
//		purposes of illustration!
//

HRESULT CMyD3DApplication::ConfirmDevice( 
				D3DCAPS9* pCaps,			// Capabilities
				DWORD dwBehavior,						// Behavioral bits
                D3DFORMAT adapterFormat, D3DFORMAT backBufferFormat )
{
		//	One shot initializer

	if (!bDump)
	{
		bDump = true;
		DumpAdapterInfo();
	}

	DumpDeviceObj( pCaps, dwBehavior, adapterFormat, backBufferFormat );

		//	WARNING: The common code is only being used to enumerate despite its 
		//		having been designed to do more. By selecting E_FAIL for all devices
		//		it will halt with a non-selection dialog at the end of the run!
		//
		//		If we actually accept one with S_OK, non-released memory warnings will
		//		occur!
#if 01							
	return E_FAIL;				// Reject!
#else							// Test for compatibility
		//	___________
		//	Simple test for Vertex 1.0 or greater!
		//		Hardware / mixed 
		
    if( (D3DCREATE_HARDWARE_VERTEXPROCESSING & dwBehavior) 
		|| (D3DCREATE_MIXED_VERTEXPROCESSING & dwBehavior) )
    {
        if( pCaps->VertexShaderVersion < D3DVS_VERSION(1,0) )
		{
            return E_FAIL;		// Reject (doesn't meet criteria!)
		}
    }

    return S_OK;				// Accept
#endif
}



//
//	Root Entry Point
//

int WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, int )
{
    CMyD3DApplication d3dApp;
	char buf[80];
	DWORD dw;

    InitCommonControls();

	if (NULL == (d3dApp.fp = fopen( "DeviceDump.txt", "wt" )))
	{
		return 0;
	}

		//	Get computer name and write to log file!

	dw = sizeof(buf);
	GetComputerName( buf, &dw );
	fprintf( d3dApp.fp, "%s\n\n", buf );

		//	Initialize Common code (Folder copied from .\\DXSDK\Samples\\C++\\Common)

    if (FAILED( d3dApp.Create( hInst ))) {}

	fclose(d3dApp.fp);

    return 0;
}

